#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "VRAMCopy.h"

static void copyToVRAMSimple(const void *src, void *dest, unsigned int numBytes)
{
    const char *srcBuf = src;
    char *dstBuf = dest;
    for (int i = 0; i < numBytes; i++)
        dstBuf[i] = srcBuf[i];
}

int main(int argc, char **argv)
{
    int i, len = 1 * 1024 * 1024 + 8;
    char *srcBuf, *dstBuf;
    clock_t start, end;

    srcBuf = malloc(len);
    if (!srcBuf) {
        fprintf(stderr, "No memory\n");
        return 1;
    }
    dstBuf = malloc(len);
    if (!dstBuf) {
        fprintf(stderr, "No memory\n");
        free(srcBuf);
        return 1;
    }
    for (i = 0; i < len; i++) srcBuf[i] = i % 256;
    printf("src %p dst %p\n", srcBuf, dstBuf);

    start = clock();
    for (i = 0; i < 10000; i++) {
        copyToVRAMSimple(srcBuf + 1, dstBuf + 1, len - 1);
    }
    end = clock();
    printf("byte loop: %g sec\n", (end - start) / (double)CLOCKS_PER_SEC);

    start = clock();
    for (i = 0; i < 10000; i++) {
        memcpy(dstBuf + 1, srcBuf + 1, len - 1);
    }
    end = clock();
    printf("memcpy: %g sec\n", (end - start) / (double)CLOCKS_PER_SEC);

    start = clock();
    for (i = 0; i < 10000; i++) {
        copyToVRAMNoAltivec(srcBuf + 1, dstBuf + 1, len - 1);
    }
    end = clock();
    printf("copyToVRAMNoAltivec: %g sec\n", (end - start) / (double)CLOCKS_PER_SEC);

    start = clock();
    for (i = 0; i < 10000; i++) {
        copyToVRAMAltivec(srcBuf + 1, dstBuf + 1, len - 1);
    }
    end = clock();
    printf("copyToVRAMAltivec: %g sec\n", (end - start) / (double)CLOCKS_PER_SEC);

    start = clock();
    for (i = 0; i < 10000; i++) {
        copyFromVRAMNoAltivec(srcBuf + 1, dstBuf + 1, len - 1);
    }
    end = clock();
    printf("copyFromVRAMNoAltivec: %g sec\n", (end - start) / (double)CLOCKS_PER_SEC);

    start = clock();
    for (i = 0; i < 10000; i++) {
        copyFromVRAMAltivec(srcBuf + 1, dstBuf + 1, len - 1);
    }
    end = clock();
    printf("copyFromVRAMAltivec: %g sec\n", (end - start) / (double)CLOCKS_PER_SEC);

    free(dstBuf);
    free(srcBuf);
    return 0;
}
